//+-------------------------------------------------------------------------
//|
//| SudokuSolver
//|
//| Implements a solving algorithm for sudoku puzzles.
//|
//| (c) 2009, 2020 Dietmar Koenig <http://www.faoileag.de>
//|
//| This program is free software: you can redistribute it and/or modify
//| it under the terms of the GNU General Public License as published by
//| the Free Software Foundation, either version 3 of the License, or
//| (at your option) any later version.
//|
//| This program is distributed in the hope that it will be useful,
//| but WITHOUT ANY WARRANTY; without even the implied warranty of
//| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//| GNU General Public License for more details.
//|
//| You should have received a copy of the GNU General Public License
//| along with this program. If not, see <https://www.gnu.org/licenses/>.
//|
//+-------------------------------------------------------------------------
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Random;

public class SudokuSolver {
  private static int instanceCounter = 0;

  private int mInstanceId = 0;

  private int[][] mGrid = new int[9][9];
  private int[] mDigitsUsed = new int[10]; // 0..9

  private int mSteps = 0;
  private int mRecursionLevel = 0;

  // You can use this to implement a brute force approach,
  // but it does not really help.
  // Valid (meaningfull) values are 2, 3..9. Value 2 is a
  // special case, then only ambiguities where there are
  // exactly 2 possibilities are resolved by recursion.
  private int mAmbiguityResolutionThreshold = 0;

  private int mResultNeutral = 0;
  private int mResultSolved = 1;
  private int mResultError = 2;
  private int mResultStuck = 3;

  //-------------------------------------------------------------------------
  // Default constructor.
  //-------------------------------------------------------------------------
  public SudokuSolver( ) {
    SudokuSolver.instanceCounter++;
    mInstanceId = SudokuSolver.instanceCounter;
    mSteps = 0;
    mRecursionLevel = 0;
  }

  //-------------------------------------------------------------------------
  // Call this to start the solving process.
  //-------------------------------------------------------------------------
  public boolean Solve( String gridstr ) {
    return Solve( gridstr, 0, 0, -1 );
  }

  //-------------------------------------------------------------------------
  // Call this to start the solving process.
  //-------------------------------------------------------------------------
  public boolean Solve( String gridstr, int steps,
                        int parent, int recursionLevel ) {
    SetGridFrom( gridstr );
    mSteps = steps;
    mRecursionLevel = recursionLevel + 1;

    System.out.println( "========================================" );
    if ( mInstanceId == 1 ) {
      PrintMsg( "solving grid:" );
      PrintBoard();
      PrintMsg( "empty spaces: " + mDigitsUsed[0] );
    }
    else {
      PrintMsg( "started by " + parent );
      PrintMsg( "recursion depth: " + mRecursionLevel );
      PrintMsg( "empty spaces: " + mDigitsUsed[0] );
      //PrintBoard();
      //System.exit( 0 );
    }

    return Run();
  }

  //-------------------------------------------------------------------------
  // Gets the number of solving steps taken so far.
  //-------------------------------------------------------------------------
  public int GetStepCount() {
    return mSteps;
  }

  //-------------------------------------------------------------------------
  // Serializes the grid into a string.
  //-------------------------------------------------------------------------
  public String GridToString( ) {
    String result = "";

    for (int i = 0; i < 9; i++) {
      for (int j = 0; j < 9; j++) {
        result += String.valueOf( mGrid[i][j] );
      }
    }

    return result;
  }
  //-------------------------------------------------------------------------
  // Here end the public methods.
  //-------------------------------------------------------------------------

  //-------------------------------------------------------------------------
  // Main loop, all solving operations are done here.
  //-------------------------------------------------------------------------
  private boolean Run( ) {
    int cycle = 0;
    int resultOfSolvingRun = 0;
    int numberOfUnsolvedSpacesBeforeOperations = mDigitsUsed[0];
    Vector gridStack = new Vector();
    
    while ( resultOfSolvingRun == mResultNeutral ) {
      cycle++;
      PrintMsg( "cycle " + cycle );
      PrintMsg( "empty spaces: " + mDigitsUsed[0] );
      PrintMsg( "random placements: " + gridStack.size() );

      resultOfSolvingRun = TryRowTripletCrossSolving();
      if ( resultOfSolvingRun == mResultNeutral ) {
        resultOfSolvingRun = TryColumnTripletCrossSolving();
        if ( resultOfSolvingRun == mResultNeutral ) {
          resultOfSolvingRun = TryQuadrantFillWithRowColumnConstraint();
          if ( resultOfSolvingRun == mResultNeutral ) {
            resultOfSolvingRun = TryRowFillWithQuadrantConstraint();
            if ( resultOfSolvingRun == mResultNeutral ) {
              resultOfSolvingRun = TryColumnFillWithQuadrantConstraint();
              if ( resultOfSolvingRun == mResultNeutral ) {
                resultOfSolvingRun = TryQuadrantBackElimination();
                if ( resultOfSolvingRun == mResultNeutral ) {
                  resultOfSolvingRun = Try8of9Solving();
                }
              }
            }
          }
        }
      }

      if ( resultOfSolvingRun == mResultNeutral ) {
        if ( mDigitsUsed[0] == numberOfUnsolvedSpacesBeforeOperations ) {
          // Still no change. Try setting some numbers randomly:
          if ( gridStack.size() > 1 ) {
            // Too many random placements. Start again with just one:
            String s = (String)gridStack.firstElement();
            SetGridFrom( s );
            gridStack.clear();
          }
          gridStack.addElement( GridToString( ) );
          resultOfSolvingRun = SetOneSpaceToPlausibleRandomNumber();
        }
      }

      if ( resultOfSolvingRun == mResultError ) {
        if ( gridStack.size() > 0 ) {
          String s = (String)gridStack.lastElement();
          gridStack.removeElementAt( gridStack.size() - 1 );
          SetGridFrom( s );
          resultOfSolvingRun = mResultNeutral;
        }
      }

      numberOfUnsolvedSpacesBeforeOperations = mDigitsUsed[0];

      PrintMsg( "solving steps: " + mSteps );
    } // end of main loop

    return EvalResultOfSolvingRun( resultOfSolvingRun );
  }

  //-------------------------------------------------------------------------
  // Evaluates the result of the solving run
  //-------------------------------------------------------------------------
  private boolean EvalResultOfSolvingRun( int resultOfSolvingRun ) {
    PrintMsg( "--- end of solving run ---");
    if ( resultOfSolvingRun == 3 ) {
      PrintMsg( "reason: stuck :-(" );
      PrintBoard();
    }
    else if ( resultOfSolvingRun == 2 ) {
      PrintMsg( "reason: grid not plausible." );
      //PrintBoard();
    }
    else if ( resultOfSolvingRun == 1 ) {
      if ( PassesIntegrityTest() ) {
        PrintMsg( "reason: SOLVED!!!" );
        if ( mInstanceId == 1 ) {
          PrintMsg( "solved grid:" );
          PrintBoard();
        }
        return true;
      }
      else {
        PrintMsg( "reason: solved, but is invalid:" );
        PrintBoard();
      }
    }
    else {
      PrintMsg( "reason: unknown (" + resultOfSolvingRun + ")" );
    }
    return false;
  }

  //-------------------------------------------------------------------------
  // Returns mResultError if no valid number exists; otherwise, mResultNeutral
  //-------------------------------------------------------------------------
  private int SetOneSpaceToPlausibleRandomNumber( ) {
    PrintMsg( "Setting one space randomly" );
    mSteps++;      

    Vector posR = new Vector();
    Vector posC = new Vector();
    for ( int r = 0; r < 9; r++ ) {
      for ( int c = 0; c < 9; c++ ) {
        if ( mGrid[r][c] == 0 ) {
          posR.addElement( new Integer( r ) );
          posC.addElement( new Integer( c ) );
        }
      }
    }

    Random rand = new Random( );

    int spaceListIndex = rand.nextInt( posR.size() );
    int r = ((Integer)posR.elementAt( spaceListIndex )).intValue();
    int c = ((Integer)posC.elementAt( spaceListIndex )).intValue();

    int n = rand.nextInt( 9 ) + 1;
    int nAtStart = n;

    while ( !PassesValidityCheck( r, c, n ) ) {
      n++;
      if ( n == 10 ) {
        n = 1;
      }
      if ( n == nAtStart ) {
        // no valid number exists for this space
        return mResultError;
      }
    }
    SetGridPos( r, c, n );
    return CheckGridStatus(); 
  }

  //-------------------------------------------------------------------------
  // Tries to fill empty spaces in a quadrant by reducing the range of
  // possible values for each empty space, ideally down to one.
  //-------------------------------------------------------------------------
  private int TryQuadrantBackElimination( ) {
    //PrintMsg( "TryQuadrantBackElimination()" );
    for ( int q = 0; q < 9; q++ ) {
      mSteps++;

      // key = index in quadrant; value = solution space
      Hashtable empties = new Hashtable();
      int[] quadrant = GetQuadrant( q );

      // Go through quadrant and remove all numbers, that are already in
      // use elsewhere in the quadrant:
      String missing = "123456789";
      for ( int i = 0; i < 9; i++ ) {
        int n = quadrant[i];
        if ( n != 0 ) {
          missing = missing.replace( String.valueOf( n ), "" );
        }
      }

      // Create a list of empties with an initial solution space:
      for ( int i = 0; i < 9; i++ ) {
        if ( quadrant[i] == 0 ) {
          empties.put( new Integer( i ), missing );
        }
      }

      // Now narrow down the solution space for each empty:
      Enumeration indexes = empties.keys(); 
      while ( indexes.hasMoreElements() ) {
        int index = ((Integer)indexes.nextElement()).intValue();
        int baseRow = GetQuadrantBaseRow( q );
        int baseCol = GetQuadrantBaseCol( q );
        int rowOffset = GetRowOffsetForQuadrantIndex( index );
        int colOffset = GetColOffsetForQuadrantIndex( index );
        int r = baseRow + rowOffset;
        int c = baseCol + colOffset;

        // Narrow down the solution space using cross checking:
        String solutionSpace = (String)empties.get( index );
        char[] nums = solutionSpace.toCharArray();
        for ( int i = 0; i < nums.length; i++ ) {
          int n = Integer.valueOf( nums[i] ) - 48;
          if (( IsInRow( r, n ) ) || ( IsInColumn( c, n ) )) {
            solutionSpace = solutionSpace.replace( String.valueOf( n ), "" );
          }
        }

        // Now do something with the narrowed down solution space:
        if ( solutionSpace.length() == 1 ) {
          int n = Integer.valueOf( solutionSpace.charAt( 0 ) ) - 48;
          SetGridPos( r, c, n );
        }
        else {
          if (( solutionSpace.length() > 2 ) &&
              ( mAmbiguityResolutionThreshold == 2 )) {
            return CheckGridStatus();
          }
          for ( int i = 0; i < solutionSpace.length(); i++ ) {
            if ( i < mAmbiguityResolutionThreshold ) {
              int n = Integer.valueOf( solutionSpace.charAt( i ) ) - 48;
              SetGridPos( r, c, n );
              if ( StartRecursion( ) ) {
                return CheckGridStatus();
              }
              SetGridPos( index, c, 0 );
            }
          }
        }
      }
    }
    return CheckGridStatus();
  }

  //-------------------------------------------------------------------------
  // Tries to reduce the solution space for empties by looking at what
  // numbers can't be a solution for them because they are already set
  // elsewhere in the quadrant the empty space belongs to.
  //-------------------------------------------------------------------------
  private int TryRowFillWithQuadrantConstraint( ) {
    //PrintMsg( "TryRowFillWithQuadrantConstraint()" );
    for ( int r = 0; r < 9; r++ ) {
      for ( int n = 1; n < 10; n++ ) {
        if ( mDigitsUsed[ n ] < 9 ) { 
          if ( !IsInRow( r, n ) ) {
            if ( TrySetNumberInRowWithQuadrantConstraint( r, n ) ) {
              return CheckGridStatus();
            }
          }
        }
      }
    }
    return CheckGridStatus();
  }

  //-------------------------------------------------------------------------
  // Tries to reduce the solution space for empties by looking at what
  // numbers can't be a solution for them because they are already set
  // elsewhere in the quadrant the empty space belongs to.
  //-------------------------------------------------------------------------
  private int TryColumnFillWithQuadrantConstraint( ) {
    //PrintMsg( "TryColumnFillWithQuadrantConstraint()" );
    for ( int c = 0; c < 9; c++ ) {
      for ( int n = 1; n < 10; n++ ) {
        if ( mDigitsUsed[ n ] < 9 ) { 
          if ( !IsInColumn( c, n ) ) {
            if ( TrySetNumberInColWithQuadrantConstraint( c, n ) ) {
              return CheckGridStatus();
            }
          }
        }
      }
    }
    return CheckGridStatus();
  }

  //-------------------------------------------------------------------------
  // Tries to reduce the solution space for empties by looking at what
  // numbers can't be a solution for them because they are already set
  // elsewhere in the same row or column the empty space belongs to.
  //-------------------------------------------------------------------------
  private int TryQuadrantFillWithRowColumnConstraint( ) {
    //PrintMsg( "TryQuadrantFillWithRowColumnConstraint()" );
    for ( int q = 0; q < 9; q++ ) {
      for ( int n = 1; n < 10; n++ ) {
        if ( mDigitsUsed[ n ] < 9 ) { 
          if ( !IsInQuadrant( q, n ) ) {
            if ( TrySettingNumberInQuadrant( q, n ) ) {
              return CheckGridStatus();
            }
          }
        }
      }
    }
    return CheckGridStatus();
  }

  //-------------------------------------------------------------------------
  // Tries to find unambigous numbers to fill in by checking againts cols.
  //-------------------------------------------------------------------------
  private int TryRowTripletCrossSolving( ) {
    //PrintMsg( "TryRowTripletCrossSolving()" );
    for (int rowGroup = 0; rowGroup < 3; rowGroup++ ) {
      for ( int rowOffset = 0; rowOffset < 3; rowOffset++ ) {
        // try this for each row of a triplet
        for ( int n = 1; n < 10; n++ ) {
          if ( mDigitsUsed[ n ] < 9 ) { 
            int r = (rowGroup * 3) + rowOffset; 
            if ( !IsInRow( r, n ) ) {
              if ( IsInOtherRows( r, n ) ) {
                if ( TrySettingNumberInRow( r, n ) ) {
                  return CheckGridStatus();
                }
              }
            }
          }
        }
      }
    }
    return CheckGridStatus();
  }

  //-------------------------------------------------------------------------
  // Tries to find unambigous numbers to fill in by checking against rows.
  //-------------------------------------------------------------------------
  private int TryColumnTripletCrossSolving( ) {
    //PrintMsg( "TryColumnTripletCrossSolving()" );
    for ( int colGroup = 0; colGroup < 3; colGroup++ ) {
      for ( int colOffset = 0; colOffset < 3; colOffset++ ) {
        // try this for each column of a triplet
        for ( int n = 1; n < 10; n++ ) {
          if ( mDigitsUsed[ n ] < 9 ) { 
            int c = (colGroup * 3) + colOffset;
            if ( !IsInColumn( c, n ) ) {
              if ( IsInOtherColumns( c, n ) ) {
                if ( TrySettingNumberInColumn( c, n ) ) {
                  return CheckGridStatus();
                }
              }
            }
          }
        }
      }
    }
    return CheckGridStatus();
  }
 
  //-------------------------------------------------------------------------
  // Tries to find rows, columns or quadrants with just one empty space. This
  // can then be easily filled.
  //-------------------------------------------------------------------------
  private int Try8of9Solving( ) {
    //PrintMsg( "Try8of9Solving()" );
    int unsolved = TrySolvingRows8of9( );
    if ( unsolved > 0 ) {
      unsolved = TrySolvingColumns8of9( );
      if ( unsolved > 0 ) {
        TrySolvingQuadrants8of9( );
      }
    }
    return CheckGridStatus();
  }

  //-------------------------------------------------------------------------
  // Returns the number of unsolved rows.
  //-------------------------------------------------------------------------
  private int TrySolvingRows8of9( ) {
    int unsolved = 9;
    for ( int i = 0; i < 9; i++ ) {
      unsolved -= SolveRow8of9( i );
      if ( unsolved <= 0 ) {
        return unsolved;
      }
    }

    return unsolved;
  }

  //-------------------------------------------------------------------------
  // Returns the number of unsolved columns.
  //-------------------------------------------------------------------------
  private int TrySolvingColumns8of9( ) {
    int unsolved = 9;

    for ( int i = 0; i < 9; i++ ) {
      unsolved -= SolveColumn8of9( i );
      if ( unsolved <= 0 ) {
        return unsolved;
      }
    }

    return unsolved;
  }

  //-------------------------------------------------------------------------
  // Returns the number of unsolved quadrants.
  //-------------------------------------------------------------------------
  private int TrySolvingQuadrants8of9( ) {
    int unsolved = 9;

    for ( int i = 0; i < 9; i++ ) {
      unsolved -= SolveQuadrant8of9( i );
      if ( unsolved <= 0 ) {
        return unsolved;
      }
    }

    return unsolved;
  }

  //-------------------------------------------------------------------------
  // For all empty spaces in the row, check if the number is in the same
  // quadrant as the empty space. Assemble a list of all spaces where it
  // is not and try setting the number for each of these spaces.
  // Returns true if the grid is solved; otherwise, false.
  //-------------------------------------------------------------------------
  private boolean TrySetNumberInRowWithQuadrantConstraint( int r, int n ) {
    mSteps++;

    int qil = 0; // index left quadrant for row
    int qim = 1; // index middle quadrant for row
    int qir = 2; // index right quadrant for row
    if ( r > 2 ) {
      qil += 3;
      qim += 3;
      qir += 3;
      if ( r > 5 ) {
        qil += 3;
        qim += 3;
        qir += 3;
      }
    }
    int[] qis = { qil, qil, qil, qim, qim, qim, qir, qir, qir };

    int numOfSuitableSpaces = 0;
    Vector suitableSpaces = new Vector(9);
    for ( int c = 0; c < 9; c++ ) {
      if (( mGrid[r][c] == 0 ) && ( !IsInQuadrant( qis[c], n ) )) {
        suitableSpaces.addElement( new Integer( c ) );
        numOfSuitableSpaces++;
      }
    }

    if ( numOfSuitableSpaces == 1 ) {
      int index = ((Integer)suitableSpaces.firstElement()).intValue();
      SetGridPos( r, index, n );
      return IsSolved();
    }

    if (( numOfSuitableSpaces > 2 ) && ( mAmbiguityResolutionThreshold == 2 )) {
      return IsSolved();
    }

    for ( int i = 0; i < numOfSuitableSpaces; i++ ) {
      if ( i < mAmbiguityResolutionThreshold ) {
        int index = ((Integer)suitableSpaces.elementAt( i )).intValue();
        SetGridPos( r, index, n );
        if ( StartRecursion( ) ) {
          return true;
        }
        SetGridPos( r, index, 0 );
      }
    }

    return IsSolved();
  }

  //-------------------------------------------------------------------------
  // For all empty spaces in the column, check if the number is in the same
  // quadrant as the empty space. Assemble a list of all spaces where it
  // is not and try setting the number for each of these spaces.
  // Returns true if the grid is solved; otherwise, false.
  //-------------------------------------------------------------------------
  private boolean TrySetNumberInColWithQuadrantConstraint( int c, int n ) {
    mSteps++;

    int qit = 0; // index top quadrant for column
    int qim = 3; // index middle quadrant for column
    int qib = 6; // index bottom quadrant for column
    if ( c > 2 ) {
      qit++;
      qim++;
      qib++;
      if ( c > 5 ) {
        qit++;
        qim++;
        qib++;
      }
    }
    int[] qis = { qit, qit, qit, qim, qim, qim, qib, qib, qib };

    int numOfSuitableSpaces = 0;
    Vector suitableSpaces = new Vector(9);
    for ( int r = 0; r < 9; r++ ) {
      if (( mGrid[r][c] == 0 ) && ( !IsInQuadrant( qis[r], n ) )) {
        suitableSpaces.addElement( new Integer( r ) );
        numOfSuitableSpaces++;
      }
    }

    if ( numOfSuitableSpaces == 1 ) {
      int index = ((Integer)suitableSpaces.firstElement()).intValue();
      SetGridPos( index, c, n );
      return IsSolved();
    }

    if (( numOfSuitableSpaces > 2 ) && ( mAmbiguityResolutionThreshold == 2 )) {
      return IsSolved();
    }

    for ( int i = 0; i < numOfSuitableSpaces; i++ ) {
      if ( i < mAmbiguityResolutionThreshold ) {
        int index = ((Integer)suitableSpaces.elementAt( i )).intValue();
        SetGridPos( index, c, n );
        if ( StartRecursion( ) ) {
          return true ;
        }
        SetGridPos( index, c, 0 );
      }
    }

    return IsSolved();
  }

  //-------------------------------------------------------------------------
  // For all empty spaces in the quadrant, check if the number is already in
  // the same row or column as the empty space. Assemble a list of all spaces
  // where it is not and try setting the number for each of these spaces.
  // Returns true if the grid is solved; otherwise, false.
  //-------------------------------------------------------------------------
  private boolean TrySettingNumberInQuadrant( int q, int n ) {
    mSteps++;

    int[] quadrant = GetQuadrant( q );

    int baseRow = GetQuadrantBaseRow( q );
    int baseCol = GetQuadrantBaseCol( q );

    int r = 0;
    int c = 0;

    int numOfSuitableSpaces = 0;
    Vector suitableSpaces = new Vector(9);
    for ( int i = 0; i < 9; i++ ) {
      if ( quadrant[i] == 0 ) {
        r = baseRow + GetRowOffsetForQuadrantIndex( i );
        c = baseCol + GetColOffsetForQuadrantIndex( i );
        if ( (!IsInRow( r, n )) && (!IsInColumn( c, n )) ) {
          suitableSpaces.addElement( new Integer( i ) );
          numOfSuitableSpaces++;
        }
      }
    }

    if ( numOfSuitableSpaces == 1 ) {
      int index = ((Integer)suitableSpaces.firstElement()).intValue();
      r = baseRow + GetRowOffsetForQuadrantIndex( index );
      c = baseCol + GetColOffsetForQuadrantIndex( index );
      SetGridPos( r, c, n );
      return IsSolved();
    }

    if (( numOfSuitableSpaces > 2 ) && ( mAmbiguityResolutionThreshold == 2 )) {
      return IsSolved();
    }

    for ( int i = 0; i < numOfSuitableSpaces; i++ ) {
      if ( i < mAmbiguityResolutionThreshold ) {
        int index = ((Integer)suitableSpaces.elementAt( i )).intValue();
        r = baseRow + GetRowOffsetForQuadrantIndex( index );
        c = baseCol + GetColOffsetForQuadrantIndex( index );
        SetGridPos( r, c, n );
        if ( StartRecursion( ) ) {
          return true ;
        }
        SetGridPos( r, c, 0 );
      }
    }

    return IsSolved();
  }

  //-------------------------------------------------------------------------
  // For all empty spaces in the row, check if the number is already in the
  // same column or quadrant as the empty space. Assemble a list of all spaces
  // where it is not and try setting the number for each of these spaces.
  // Returns true if the grid is solved; otherwise, false.
  //-------------------------------------------------------------------------
  private boolean TrySettingNumberInRow( int r, int n ) {
    mSteps++;

    int numOfSuitableSpaces = 0;
    Vector suitableSpaces = new Vector(9);
    for ( int c = 0; c < 9; c++ ) {
      if ( mGrid[r][c] == 0 ) {
        if ( !IsInColumn( c, n ) ) {
          if ( !IsInQuadrant( GetQuadrantIndexFor( r, c ), n ) ) {
            suitableSpaces.addElement( new Integer( c ) );
            numOfSuitableSpaces++;
          }
        }
      }
    }

    if ( numOfSuitableSpaces == 1 ) {
      int c = ((Integer)suitableSpaces.firstElement()).intValue();
      SetGridPos( r, c, n );
      return IsSolved();
    }

    if (( numOfSuitableSpaces > 2 ) && ( mAmbiguityResolutionThreshold == 2 )) {
      return IsSolved();
    }

    for ( int i = 0; i < numOfSuitableSpaces; i++ ) {
      if ( i < mAmbiguityResolutionThreshold ) {
        int c = ((Integer)suitableSpaces.elementAt( i )).intValue();
        SetGridPos( r, c, n );
        if ( StartRecursion( ) ) {
          return true;
        }
        SetGridPos( r, c, 0 );
      }
    }

    return IsSolved();
  }

  //-------------------------------------------------------------------------
  // For all empty spaces in the column, check if the number is already in
  // the same row or quadrant as the empty space. Assemble a list of all
  // spaces where it is not and try setting the number for each of these
  // spaces.
  // Returns true if the grid is solved; otherwise, false.
  //-------------------------------------------------------------------------
  private boolean TrySettingNumberInColumn( int c, int n ) {
    mSteps++;

    int numOfSuitableSpaces = 0;
    Vector suitableSpaces = new Vector(9);
    for ( int r = 0; r < 9; r++ ) {
      if ( mGrid[r][c] == 0 ) {
        if ( !IsInRow( r, n ) ) {
          if ( !IsInQuadrant( GetQuadrantIndexFor( r, c ), n ) ) {
            suitableSpaces.addElement( new Integer( r ) );
            numOfSuitableSpaces++;
          }
        }
      }
    }

    if ( numOfSuitableSpaces == 1 ) {
      int r = ((Integer)suitableSpaces.firstElement()).intValue();
      SetGridPos( r, c, n );
      return IsSolved();
    }

    if (( numOfSuitableSpaces > 2 ) && ( mAmbiguityResolutionThreshold == 2 )) {
      return IsSolved();
    }

    for ( int i = 0; i < numOfSuitableSpaces; i++ ) {
      if ( i < mAmbiguityResolutionThreshold ) {
        int r = ((Integer)suitableSpaces.elementAt( i )).intValue();
        SetGridPos( r, c, n );
        if ( StartRecursion( ) ) {
          return true;
        }
        SetGridPos( r, c, 0 );
      }
    }

    return IsSolved();
  }

  //-------------------------------------------------------------------------
  // If there is only one empty space in the row, it can be easily filled,
  // because the number can directly be deduced from all the other numbers
  // in the row.
  // Returns 42 if row is corrupt; 1 if row is complete; otherwise, 0.
  //-------------------------------------------------------------------------
  private int SolveRow8of9( int r ) {
    mSteps++;

    int cellValue = 0;
    int indexOfSpace = -1;
    int rowSum = 0;

    for ( int c = 0; c < 9; c++ ) {
      cellValue = mGrid[r][c];
      if ( cellValue == 0 ) {
        if ( indexOfSpace != -1 ) {
          return 0;
        }
        indexOfSpace = c;
      }
      rowSum += cellValue;
    }

    int ninthNumber = 45 - rowSum;
    if (( ninthNumber < 0 ) || ( ninthNumber > 9 )) {
      // broken grid; make sure this is noticed:
      return 42; // makes unsolvedRows negative
    }

    if ( indexOfSpace != -1 ) {
      SetGridPos( r, indexOfSpace, ninthNumber );
    }

    return 1;
  }

  //-------------------------------------------------------------------------
  // If there is only one empty space in the column, it can be easily filled,
  // because the number can directly be deduced from all the other numbers
  // in the column.
  // Returns 42 if column is corrupt; 1 if column is complete; otherwise, 0.
  //-------------------------------------------------------------------------
  private int SolveColumn8of9( int c ) {
    mSteps++;

    int cellValue = 0;
    int indexOfSpace = -1;
    int colSum = 0;

    for ( int r = 0; r < 9; r++ ) {
      cellValue = mGrid[r][c];
      if ( cellValue == 0 ) {
        if ( indexOfSpace != -1 ) {
          return 0;
        }
        indexOfSpace = r;
      }
      colSum += cellValue;
    }

    int ninthNumber = 45 - colSum;
    if (( ninthNumber < 0 ) || ( ninthNumber > 9 )) {
      // broken grid; make sure this is noticed:
      return 42; // makes unsolvedCols negative
    }

    if ( indexOfSpace != -1 ) {
      SetGridPos( indexOfSpace, c, ninthNumber );
    }

    return 1;
  }

  //-------------------------------------------------------------------------
  // If there is only one empty space in the quadrant, it can be easily
  // filled, because the number can directly be deduced from all the other
  // numbers in the quadrant.
  // Returns 42 if quadrant is corrupt; 1 if it is complete; otherwise, 0.
  //-------------------------------------------------------------------------
  private int SolveQuadrant8of9( int q ) {
    mSteps++;

    int[] quadrant = GetQuadrant( q );

    int cellValue = 0;
    int indexOfSpace = -1;
    int quadSum = 0;

    for ( int i = 0; i < 9; i++ ) {
      cellValue = quadrant[i];
      if ( cellValue == 0 ) {
        if ( indexOfSpace != -1 ) {
          return 0;
        }
        indexOfSpace = i;
      }
      else {
        quadSum += cellValue;
      }
    }

    int ninthNumber = 45 - quadSum;
    if (( ninthNumber < 0 ) || ( ninthNumber > 9 )) {
      // broken grid; make sure this is noticed:
      return 42; // makes unsolvedQuads negative
    }

    if ( indexOfSpace != -1 ) {
      int baseRow = GetQuadrantBaseRow( q );
      int baseCol = GetQuadrantBaseCol( q );
      int r = baseRow + GetRowOffsetForQuadrantIndex( indexOfSpace );
      int c = baseCol + GetColOffsetForQuadrantIndex( indexOfSpace );
      SetGridPos( r, c, ninthNumber ); 
    }

    return 1;
  }

  //-------------------------------------------------------------------------
  // Returns true if the number is part of the row.
  //-------------------------------------------------------------------------
  private boolean IsInRow( int r, int n ) {
    for ( int c = 0; c < 9; c++ ) {
      if ( mGrid[r][c] == n ) {
        return true;
      }
    }
    return false;
  }

  //-------------------------------------------------------------------------
  // Returns true if the number is part of the column.
  //-------------------------------------------------------------------------
  private boolean IsInColumn( int c, int n ) {
    for ( int r = 0; r < 9; r++ ) {
      if ( mGrid[r][c] == n ) {
        return true;
      }
    }
    return false;
  }

  //-------------------------------------------------------------------------
  // Returns true if the number is part of the quadrant.
  //-------------------------------------------------------------------------
  private boolean IsInQuadrant( int q, int n ) {
    int[] quadrant = GetQuadrant( q );
    for ( int i = 0; i < 9; i++ ) {
      if ( quadrant[i] == n ) {
        return true;
      }
    }
    return false;
  }

  //-------------------------------------------------------------------------
  // Returns true if the number is part of the other two rows in the group.
  //-------------------------------------------------------------------------
  private boolean IsInOtherRows( int r, int n ) {
    int or1 = -1;
    int or2 = -1;
    if (( r == 0 ) || ( r == 3 ) || ( r == 6 )) {
      or1 = r + 1;
      or2 = r + 2;
    }
    else if (( r == 1 ) || ( r == 4 ) || ( r == 7 )) {
      or1 = r - 1;
      or2 = r + 1;
    }
    else if (( r == 2 ) || ( r == 5 ) || ( r == 8 )) {
      or1 = r - 1;
      or2 = r - 2;
    }
    return ( (IsInRow( or1, n )) && (IsInRow( or2, n )) );
  }

  //-------------------------------------------------------------------------
  // Returns true if the number is part of the other two columns in the group
  //-------------------------------------------------------------------------
  private boolean IsInOtherColumns( int c, int n ) {
    int oc1 = -1;
    int oc2 = -1;
    if (( c == 0 ) || ( c == 3 ) || ( c == 6 )) {
      oc1 = c + 1;
      oc2 = c + 2;
    }
    else if (( c == 1 ) || ( c == 4 ) || ( c == 7 )) {
      oc1 = c - 1;
      oc2 = c + 1;
    }
    else if (( c == 2 ) || ( c == 5 ) || ( c == 8 )) {
      oc1 = c - 1;
      oc2 = c - 2;
    }
    return ( (IsInColumn( oc1, n )) && (IsInColumn( oc2, n )) );
  }

  //-------------------------------------------------------------------------
  // Assembles a vector that contains the values of the quadrant.
  //-------------------------------------------------------------------------
  private int[] GetQuadrant( int q ) {
    int baseRow = GetQuadrantBaseRow( q );
    int baseCol = GetQuadrantBaseCol( q );
    
    int[] quadrant = new int[9];

    for ( int i = 0; i < 3; i++ ) {
      for ( int j = 0; j < 3; j++ ) {
        quadrant[(i*3) + j] = mGrid[ baseRow + i ][ baseCol + j ];
      }
    }

    return quadrant;
  }

  //-------------------------------------------------------------------------
  // Returns the base row for the quadrant.
  //-------------------------------------------------------------------------
  private int GetQuadrantBaseRow( int q ) {
    int baseRow = 0;
    if (q > 2) {
      baseRow = 3;
    }
    if (q > 5) {
      baseRow = 6;
    }
    return baseRow;
  }

  //-------------------------------------------------------------------------
  // Returns the base column for the quadrant.
  //-------------------------------------------------------------------------
  private int GetQuadrantBaseCol( int q ) {
    int baseCol = 0;
    if ( (q == 1) || (q == 4) || (q == 7) ){
      baseCol = 3;
    }
    if ( (q == 2) || (q == 5) || (q == 8) ){
      baseCol = 6;
    }
    return baseCol;
  }

  //-------------------------------------------------------------------------
  // Returns the offset from the quadrant's base row for the given index.
  //-------------------------------------------------------------------------
  private int GetRowOffsetForQuadrantIndex( int index ) {
    int rowOffset = 0;
    if ( index > 2 ) {
      rowOffset++;
    }
    if ( index > 5 ) {
      rowOffset++;
    }
    return rowOffset;
  }

  //-------------------------------------------------------------------------
  // Returns the offset from the quadrant's base column for the given index.
  //-------------------------------------------------------------------------
  private int GetColOffsetForQuadrantIndex( int index ) {
    int colOffset = 0;
    if (( index == 1 ) || ( index == 4 ) || ( index == 7 )) {
      colOffset = 1;
    }
    if (( index == 2 ) || ( index == 5 ) || ( index == 8 )) {
      colOffset = 2;
    }
    return colOffset;
  }

  //-------------------------------------------------------------------------
  // Gets the quadrant number for a given row and column.
  //-------------------------------------------------------------------------
  private int GetQuadrantIndexFor( int row, int col ) {
    int quadrantIndex = 0;
    if ( row > 2 ) {
      quadrantIndex = 3;
    }
    if ( row > 5 ) {
      quadrantIndex = 6;
    }
    if ( col > 2 ) {
      quadrantIndex++;
    }
    if ( col > 5 ) {
      quadrantIndex++;
    }
    return quadrantIndex;
  }

  //-------------------------------------------------------------------------
  // Prints the board. Spaces are represented by zeroes.
  //-------------------------------------------------------------------------
  private void PrintBoard( ) {
    System.out.println( "--------------------"); 
    for (int i = 0; i < 9; i++) {
      for (int j = 0; j < 9; j++) {
        System.out.print( " " + mGrid[i][j] );
      }
      System.out.println();
    }
    System.out.println( "--------------------"); 
  }

  //-------------------------------------------------------------------------
  // Determines if setting the number at row/column does not break any rule.
  //-------------------------------------------------------------------------
  private boolean PassesValidityCheck( int r, int c, int n ) {
    if ( mDigitsUsed[ n ] == 9 ) {
      return false;
    }
    if ( IsInRow( r, n ) ) {
      return false;
    }
    if ( IsInColumn( c, n ) ) {
      return false;
    }
    if ( IsInQuadrant( GetQuadrantIndexFor( r, c ), n ) ) {
      return false;
    }
    return true;
  }

  //-------------------------------------------------------------------------
  // Checks the current status of the grid and returns the appropriate result
  // code.
  //-------------------------------------------------------------------------
  private int CheckGridStatus() {
    //PrintMsg( "grid status: " + mDigitsUsed[0] + " empty spaces left" );
    if ( !GridIsPlausible() ) {
      return mResultError;
    }
    else if ( IsSolved() ) {
      return mResultSolved;
    }
    return mResultNeutral;
  }

  //-------------------------------------------------------------------------
  // Very simple test - doesn't work if the counter var is corrupted.
  //-------------------------------------------------------------------------
  private boolean IsSolved( ) {
    return ( mDigitsUsed[0] == 0 );
  }

  //-------------------------------------------------------------------------
  // Determines if the grid is still plausible: i.e. no rules are broken
  //-------------------------------------------------------------------------
  private boolean GridIsPlausible( ) {
    for (int r = 0; r < 9; r++) {
      int[] digitCounts = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
      for (int c = 0; c < 9; c++) {
        int d = mGrid[r][c];
        if (( d < 0 ) || ( d > 9 )) {
          PrintMsg( "not plausible: fault at row " + r + " col " + c );
          //PrintBoard();
          return false;
        }
        digitCounts[d]++;
        if ( ( d > 0 ) && ( digitCounts[d] > 1 ) ) {
          PrintMsg( "not plausible: fault at row " + r );
          //PrintBoard();
          return false;
        }
      }
    }

    for (int c = 0; c < 9; c++) {
      int[] digitCounts = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
      for (int r = 0; r < 9; r++) {
        int d = mGrid[r][c];
        digitCounts[d]++;
        if ( ( d > 0 ) && ( digitCounts[d] > 1 ) ) {
          PrintMsg( "not plausible: fault at column " + c );
          //PrintBoard();
          return false;
        }
      }
    }
    return true;
  }

  //-------------------------------------------------------------------------
  // Determines if solution is correct, i.e. it's plausible and sums are 45
  //-------------------------------------------------------------------------
  private boolean PassesIntegrityTest( ) {
    if ( !GridIsPlausible() ) {
      return false;
    }
  
    // Test for row sums:
    for (int r = 0; r < 9; r++) {
      int rowSum = 0;
      for (int c = 0; c < 9; c++) {
        rowSum += mGrid[r][c];
      }
      if ( rowSum != 45 ) {
        PrintMsg( "failed sum test on row " + r + " with " + rowSum);
        //PrintBoard();
        return false;
      }
    }

    // Test for column sums:
    for (int c = 0; c < 9; c++) {
      int colSum = 0;
      for (int r = 0; r < 9; r++) {
        colSum += mGrid[r][c];
      }
      if ( colSum != 45 ) {
        PrintMsg( "failed sum test on col " + c + " with " + colSum);
        //PrintBoard();
        return false;
      }
    }

    // Test for quadrant sums:
    for (int q = 0; q < 9; q++) {
      int[] quadrant = GetQuadrant( q );
      int quadSum = 0;
      for (int i = 0; i < 9; i++) {
        quadSum += quadrant[i];
      }
      if ( quadSum != 45 ) {
        PrintMsg( "failed sum test on quadrant " + q + " with " + quadSum);
        //PrintBoard();
        return false;
      }
    }
    return true;
  }

  //-------------------------------------------------------------------------
  // Starts a new instance of SudokuSolver with the current grid as start.
  //-------------------------------------------------------------------------
  private boolean StartRecursion( ) {
    SudokuSolver subSolver = new SudokuSolver( );
    boolean result = subSolver.Solve( GridToString(), mSteps,
                                      mInstanceId, mRecursionLevel );
    mSteps = subSolver.GetStepCount();
    if ( result ) {
      SetGridFrom( subSolver.GridToString( ) );
    }
    return result;
  }

  //-------------------------------------------------------------------------
  // Sets the grid to the values provided by the string
  //-------------------------------------------------------------------------
  public void SetGridFrom( String gridstr ) {
    // Reset counters:
    for ( int i = 0; i < 10; i++ ) {
      mDigitsUsed[i] = 0;
    }

    for (int i = 0; i < 9; i++) {
      for (int j = 0; j < 9; j++) {
        int d = Character.digit( gridstr.charAt( (i*9) + j ), 10 );
        mGrid[i][j] = d;
        mDigitsUsed[d]++;
      }
    }
  }

  //-------------------------------------------------------------------------
  // Updates the counter for the used digits as well.
  //-------------------------------------------------------------------------
  private void SetGridPos( int r, int c, int n ) {
    try {
      if ( n == 0 ) {
        mDigitsUsed[mGrid[r][c]]--;
      }
      else {
        mDigitsUsed[0]--;
      }
      mGrid[r][c] = n;
      mDigitsUsed[n]++;
    }
    catch ( java.lang.ArrayIndexOutOfBoundsException e ) {
      PrintMsg( "r=" + r + " c=" + c + " n=" + n );
      PrintBoard();
      throw e;
    }
  }

  //-------------------------------------------------------------------------
  // Formatted string output.
  //-------------------------------------------------------------------------
  private void PrintMsg( String msg ) {
    System.out.println( "SudokuSolver " + mInstanceId + ": " + msg );
  }
}

